
#include <stdio.h>
#include <sys/types.h>
#include <sys/stat.h>
#include <sys/ioctl.h>
#include <sys/time.h>
#include <fcntl.h>
#include <errno.h>
#include <unistd.h>
#include <string.h>
#include "drisync.h"

typedef enum {
    _DRM_VBLANK_ABSOLUTE = 0x0,		/* Wait for specific vblank sequence number */
    _DRM_VBLANK_RELATIVE = 0x1,		/* Wait for given number of vblanks */
    _DRM_VBLANK_SIGNAL   = 0x40000000	/* Send signal instead of blocking */
} drm_vblank_seq_type_t;

#define DRM_IOCTL_BASE			'd'
#define DRM_IOWR(nr,type)		_IOWR(DRM_IOCTL_BASE,nr,type)
#define _DRM_VBLANK_FLAGS_MASK _DRM_VBLANK_SIGNAL

struct drm_wait_vblank_request {
	drm_vblank_seq_type_t type;
	unsigned int sequence;
	unsigned long signal;
};

struct drm_wait_vblank_reply {
	drm_vblank_seq_type_t type;
	unsigned int sequence;
	long tval_sec;
	long tval_usec;
};

typedef union drm_wait_vblank {
	struct drm_wait_vblank_request request;
	struct drm_wait_vblank_reply reply;
} drm_wait_vblank_t;

#define DRM_IOCTL_WAIT_VBLANK		DRM_IOWR(0x3a, drm_wait_vblank_t)

typedef enum {
    DRM_VBLANK_ABSOLUTE = 0x0,		/* Wait for specific vblank sequence number */
    DRM_VBLANK_RELATIVE = 0x1,		/* Wait for given number of vblanks */
    DRM_VBLANK_SIGNAL   = 0x40000000	/* Send signal instead of blocking */
} drmVBlankSeqType;

typedef struct _drmVBlankReq {
	drmVBlankSeqType type;
	unsigned int sequence;
	unsigned long signal;
} drmVBlankReq, *drmVBlankReqPtr;

typedef struct _drmVBlankReply {
	drmVBlankSeqType type;
	unsigned int sequence;
	long tval_sec;
	long tval_usec;
} drmVBlankReply, *drmVBlankReplyPtr;

typedef union _drmVBlank {
	drmVBlankReq request;
	drmVBlankReply reply;
} drmVBlank, *drmVBlankPtr;

static int dri_fd;
const char *dri_dev = "/dev/dri/card0";

int drmWaitVBlank( int fd, drmVBlankPtr vbl )
{
    int ret;

    do {
       ret = ioctl( fd, DRM_IOCTL_WAIT_VBLANK, vbl );
       vbl->request.type &= ~DRM_VBLANK_RELATIVE;
    } while (ret && errno == EINTR);

    return ret;
}

int drisync_init( void )
{
    drmVBlank blank;
    int ret;

    dri_fd = open( dri_dev, O_RDWR );
    if( dri_fd < 0 ) {
        fprintf( stderr, "drisync: Can't open device %s: %s\n", dri_dev, strerror( errno ) );
        return 0;
    }

    blank.request.type = DRM_VBLANK_RELATIVE;
    blank.request.sequence = 1;
    if( (ret = drmWaitVBlank( dri_fd, &blank )) ) {
        fprintf( stderr, "drisync: Error %d from VBLANK ioctl, not supported by your DRI driver?\n", ret );
        close( dri_fd );
        return 0;
    }
    return 1;
}

void drisync_shutdown( void )
{
    close( dri_fd );
}

void drisync_wait_for_retrace( void )
{
    drmVBlank blank;

    blank.request.type = DRM_VBLANK_RELATIVE;
    blank.request.sequence = 1;
    drmWaitVBlank( dri_fd, &blank );
}

